/*******************************************************************************
 * Copyright (c) 2009-2013 CWI
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
*******************************************************************************/
package org.rascalmpl.eclipse.library.vis.figure.interaction;

import static org.rascalmpl.eclipse.library.vis.util.vector.Dimension.HOR_VER;

import org.rascalmpl.eclipse.library.vis.figure.Figure;
import org.rascalmpl.eclipse.library.vis.figure.combine.WithInnerFig;
import org.rascalmpl.eclipse.library.vis.properties.PropertyManager;
import org.rascalmpl.eclipse.library.vis.properties.PropertyValue;
import org.rascalmpl.eclipse.library.vis.swt.IFigureConstructionEnv;
import org.rascalmpl.eclipse.library.vis.util.NameResolver;
import org.rascalmpl.eclipse.library.vis.util.vector.Dimension;

public class FigureSwitch extends WithInnerFig{

	Figure[] choices;
	PropertyValue<Integer> choice;
	boolean visible;
	
	public FigureSwitch(PropertyValue<Integer> choice, Figure[] choices, PropertyManager properties) {
		super(null, properties);
		this.choices = choices;
		this.choice = choice;
	}

	@Override
	public void setChildren(IFigureConstructionEnv env, NameResolver resolver){
		children = choices;
	}
	
	@Override
	public void initElem(IFigureConstructionEnv env, MouseOver mparent, boolean swtSeen, boolean visible, NameResolver resolver){
		this.visible = visible;
	}

	@Override
	public void computeMinSize() {
		minSize.set(0,0);
		resizable.set(false,false);
		for(Figure fig : choices){
			setInnerFig(fig);
			super.computeMinSize();
			minSize.setMax(fig.minSize);
			for(Dimension d : HOR_VER){
				resizable.set(d,resizable.get(d) || fig.resizable.get(d));
			}
		}
	}
	

	public boolean initChildren(IFigureConstructionEnv env,
			NameResolver resolver, MouseOver mparent, boolean swtSeen, boolean visible) {
		
		boolean swtSeenResult = false;
		for(int i = 0; i < children.length ; i++){
			boolean nextVisible = visible && i == choice.getValue();
			boolean swt = children[i].init(env, resolver,mparent, swtSeen, nextVisible);
			if(nextVisible){
				swtSeenResult = swt;
			}
		}
		return swtSeenResult;
	}
	
	public void finalize(boolean needsRecompute){
		if(visible){
			setInnerFig(choices[choice.getValue()]);
		}
	}
	
	@Override
	public void hideElement(IFigureConstructionEnv env) {
		for(int i = 0 ; i < choices.length ; i++){
				choices[i].hide(env);
		}
	}
	
	@Override	
	public void destroyElement(IFigureConstructionEnv env) {
		for(int i = 0 ; i < choices.length ; i++){
			if(i != choice.getValue()){
				choices[i].destroy(env);
			}
		}
	}

}
